@@ -43,3 +43,18 @@ struct udev *udev_unref(struct udev *udev)
     free(udev);
     return NULL;
 }
+
+void udev_set_log_fn(struct udev *udev, void (*log_fn)(struct udev *udev,
+            int priority, const char *file, int line, const char *fn,
+            const char *format, va_list args))
+{
+}
+
+void udev_set_log_priority(struct udev *udev, int priority)
+{
+}
+
+int udev_get_log_priority(struct udev *udev)
+{
+    return 0;
+}
